"use strict";

var AbstractMarker = require('./AbstractMarker');

var logger = require('../../logger');

/**
 * This class describes Google maps marker (connected to {@link Alias}) that is
 * visualized on the map.
 */
function AliasMarker(params) {
  AbstractMarker.call(this, params);
}

AliasMarker.prototype = Object.create(AbstractMarker.prototype);
AliasMarker.prototype.constructor = AliasMarker;

/**
 * Returns {@link Alias} data for this marker.
 *
 * @returns {@link Alias} data for this marker
 */
AliasMarker.prototype.getAliasData = function () {
  return this._aliasData;
};

/**
 * Sets {@link Alias} data for this marker.
 *
 * @param data
 *          {@link Alias} data for this marker
 */
AliasMarker.prototype.setAliasData = function (data) {
  this._aliasData = data;
};

/**
 * Returns coordinates where marker is pointing.
 *
 * @returns {google.maps.Point} - coordinates where marker is pointing
 */
AliasMarker.prototype.getCoordinates = function () {
  var alias = this.getAliasData();
  return new google.maps.Point(alias.getX() + alias.getWidth() / 2, alias.getY() + alias.getHeight() / 2);
};

AliasMarker.prototype.init = function () {
  var self = this;
  var model = self.getCustomMap().getModel();
  return model.getByIdentifiedElement(self.getIdentifiedElement()).then(function (aliasData) {
    self.setAliasData(aliasData);
    return self._init();
  }).then(function () {
    self.show();
  });
};

module.exports = AliasMarker;
